@@ -228,7 +228,6 @@ __author__ = 'Ahmet Bakan'
 __copyright__ = 'Copyright (C) 2010-2012 Ahmet Bakan'
 
 from time import time
-from types import NoneType
 
 import numpy as np
 
@@ -269,101 +268,17 @@ def checkLabel(label):
 
     if not (''.join(label.split('_'))).isalnum():
         raise ValueError('label may contain alphanumeric characters and '
-                         'underscore, {0:s} is not valid'.format(label))
+                         'underscore, {0} is not valid'.format(label))
 
     if isReserved(label):
-        raise ValueError('{0:s} is a reserved word and cannot be used '
+        raise ValueError('{0} is a reserved word and cannot be used '
                            'as a label'.format(repr(label)))
 
     if label in READONLY:
-        raise AttributeError('{0:s} is read-only'.format(label))
+        raise AttributeError('{0} is read-only'.format(label))
 
     return label
 
-class AtomGroupMeta(type):
-
-    def __init__(cls, name, bases, dict):
-    
-        for fname, field in ATOMIC_FIELDS.items():
-
-            meth = field.meth_pl
-            getMeth = 'get' + meth
-            setMeth = 'set' + meth
-            if field.call:
-                # Define public method for retrieving a copy of data array
-                if not field.private:
-                    def getData(self, var=fname, call=field.call):
-                        try:
-                            return self._data[var].copy()
-                        except KeyError:
-                            [getattr(self, meth)() for meth in call]
-                            return self._data[var].copy()
-                # Define private method for retrieving actual data array
-                def _getData(self, var=fname, call=field.call):
-                    try:
-                        return self._data[var]
-                    except KeyError:
-                        [getattr(self, meth)() for meth in call]
-                        return self._data[var].copy()
-            else:
-                if not field.private:
-                    def getData(self, var=fname):
-                        try:
-                            return self._data[var].copy()
-                        except KeyError: 
-                            pass
-                def _getData(self, var=fname):
-                    return self._data.get(var)
-
-            if not field.private:
-                getData = wrapGetMethod(getData)
-                getData.__name__ = getMeth
-                getData.__doc__ = field.getDocstr('get')
-                setattr(cls, getMeth, getData)
-            
-            _getData = wrapGetMethod(_getData)
-            _getData.__name__ = '_' + getMeth
-            _getData.__doc__ = field.getDocstr('_get')
-            setattr(cls, '_' + getMeth, _getData)
-            
-            if field.readonly or field.private:
-                continue
-            
-            # Define public method for setting values in data array
-            def setData(self, array, var=fname, dtype=field.dtype, 
-                        ndim=field.ndim, none=field.none, flags=field.flags):
-                if array is None:
-                    self._data.pop(var, None)
-                else:
-                    if self._n_atoms == 0:
-                        self._n_atoms = len(array)
-                    elif len(array) != self._n_atoms:
-                        raise ValueError('length of array must match number '
-                                           'of atoms')
-                        
-                    if isinstance(array, list):
-                        array = np.array(array, dtype)
-                    elif not isinstance(array, np.ndarray):
-                        raise TypeError('array must be an ndarray or a list')
-                    elif array.ndim != ndim:
-                            raise ValueError('array must be {0:d} '
-                                               'dimensional'.format(ndim))
-                    elif array.dtype != dtype:
-                        try:
-                            array = array.astype(dtype)
-                        except ValueError:
-                            raise ValueError('array cannot be assigned type '
-                                             '{0:s}'.format(dtype))
-                    self._data[var] = array
-                    if none: self._none(none)
-                    if flags and self._flags:
-                        self._resetFlags(var)
-                    
-            setData = wrapSetMethod(setData)
-            setData.__name__ = setMeth 
-            setData.__doc__ = field.getDocstr('set')
-            setattr(cls, setMeth, setData)
-
 
 class AtomGroup(Atomic):
     
@@ -427,8 +342,6 @@ class AtomGroup(Atomic):
     of all coordinate sets, otherwise *C* will have a single coordinate set,
     which is a copy of of active coordinate sets of *A* and *B*."""
     
-    __metaclass__ = AtomGroupMeta
-    
     __slots__ = ['_title', '_n_atoms', '_coords', '_hv', '_sn2i', 
                  '_timestamps', '_kdtrees', '_bmap', '_bonds', '_cslabels',
                  '_acsi', '_n_csets', '_data', '_fragments',
@@ -461,14 +374,14 @@ class AtomGroup(Atomic):
 
         n_csets = self._n_csets
         if n_csets == 1:
-            return '<AtomGroup: {0:s} ({1:d} atoms)>'.format(
+            return '<AtomGroup: {0} ({1} atoms)>'.format(
                     self._title, self._n_atoms)
         elif n_csets > 1:
-            return ('<AtomGroup: {0:s} ({1:d} atoms; active #{2:d} of {3:d}' 
+            return ('<AtomGroup: {0} ({1} atoms; active #{2} of {3}' 
                     ' coordsets)>').format(self._title, self._n_atoms, 
                                            self._acsi, n_csets)
         else:
-            return '<AtomGroup: {0:s} ({1:d} atoms; no coordinates)>'.format(
+            return '<AtomGroup: {0} ({1} atoms; no coordinates)>'.format(
                     self._title, self._n_atoms)
         
     def __str__(self):
@@ -492,7 +405,7 @@ class AtomGroup(Atomic):
             if len(index):
                 if start > stop:
                     index = index[::-1]
-                selstr = 'index {0:d}:{1:d}:{2:d}'.format(start, stop, step)
+                selstr = 'index {0}:{1}:{2}'.format(start, stop, step)
                 return Selection(self, index, selstr, acsi, unique=True)
         
         elif isinstance(index, (list, np.ndarray)):
@@ -515,8 +428,8 @@ class AtomGroup(Atomic):
     def __add__(self, other):
         
         if not isinstance(other, AtomGroup):
-            raise TypeError('unsupported operand type(s) for +: {0:s} and '
-                            '{1:s}'.format(repr(type(self).__name__), 
+            raise TypeError('unsupported operand type(s) for +: {0} and '
+                            '{1}'.format(repr(type(self).__name__), 
                                            repr(type(other).__name__)))
                                                        
         new = AtomGroup(self._title + ' + ' + other._title)
@@ -524,15 +437,15 @@ class AtomGroup(Atomic):
             if self._n_csets == other._n_csets:
                 new.setCoords(np.concatenate((self._coords, other._coords), 1))
                 if self._n_csets > 1:
-                    LOGGER.info('All {0:d} coordinate sets are copied to '
-                                '{1:s}.'.format(self._n_csets, new.getTitle()))
+                    LOGGER.info('All {0} coordinate sets are copied to '
+                                '{1}.'.format(self._n_csets, new.getTitle()))
             else:
                 new.setCoords(np.concatenate((self._getCoords(), 
                                               other._getCoords())))
-                LOGGER.info('Active coordinate sets are copied to {0:s}.'
+                LOGGER.info('Active coordinate sets are copied to {0}.'
                             .format(new.getTitle()))
         elif other._n_csets:
-            LOGGER.warn('No coordinate sets are copied to {0:s}'
+            LOGGER.warn('No coordinate sets are copied to {0}'
                         .format(new.getTitle()))
         
         for key in set(list(self._data) + list(other._data)):
@@ -691,7 +604,7 @@ class AtomGroup(Atomic):
                 coords = np.array(coords)
         else:
             if coords is None:
-                raise ValueError('coordinates of {0:s} are not set'
+                raise ValueError('coordinates of {0} are not set'
                                  .format(str(atoms)))
         
         try:
@@ -732,7 +645,7 @@ class AtomGroup(Atomic):
                 self._coords = coords
                 self._n_csets = n_csets = shape[0]
 
-                if isinstance(label, (NoneType, str)):
+                if label is None or isinstance(label, str):
                     self._cslabels = [label] * n_csets
                 
                 elif isinstance(label, (list, tuple)):
@@ -774,7 +687,7 @@ class AtomGroup(Atomic):
             pass
         else:
             if coords is None:
-                raise ValueError('coordinates of {0:s} are not set'
+                raise ValueError('coordinates of {0} are not set'
                                  .format(str(atoms)))
 
         try:
@@ -794,7 +707,7 @@ class AtomGroup(Atomic):
         self._timestamps[:len(timestamps)] = timestamps
         self._timestamps[len(timestamps):] = time()
         self._kdtrees.extend([None] * diff)
-        if isinstance(label, (str, NoneType)):
+        if label is None or isinstance(label, str):
             self._cslabels.extend([label] * diff)
         elif isinstance(label, (list, tuple)):
             if len(label) == diff:
@@ -995,13 +908,6 @@ class AtomGroup(Atomic):
             else:
                 return getattr(self, '_get' + field.meth_pl)()
 
-    def isData(self, label):
-        """Deprecated, use :meth:`isDataLabel` instead."""
-        
-        from prody import deprecate
-        deprecate('isData', 'isDataLabel', 'v1.3')
-        return self.isDataLabel(label)
-        
     def isDataLabel(self, label):
         """Return **True** if data associated with *label* is present."""
         
@@ -1018,8 +924,8 @@ class AtomGroup(Atomic):
         user provided data."""
 
         if str(which).startswith('u'): # user
-            labels = [key for key in (self._data or {}).keys() 
-                        if not key in ATOMIC_FIELDS]
+            labels = [key for key in (self._data or {}) 
+                      if not key in ATOMIC_FIELDS]
         else:
             labels = list(self._data or [])
         labels.sort()
@@ -1137,8 +1043,8 @@ class AtomGroup(Atomic):
             labels.update(FLAG_PLANTERS)
             labels = list(labels)
         elif which.startswith('u'): # user
-            labels = [key for key in (self._flags or {}).keys() 
-                        if not key in FLAG_PLANTERS]
+            labels = [key for key in (self._flags or {}) 
+                      if not key in FLAG_PLANTERS]
         else:
             labels = list(self._flags or [])
         labels.sort()
@@ -1196,7 +1102,7 @@ class AtomGroup(Atomic):
             
             indices = sn2i[serial:stop:step]
             indices = indices[indices > -1]
-            return Selection(self, indices, 'serial {0:d}:{1:d}:{2:d}'
+            return Selection(self, indices, 'serial {0}:{1}:{2}'
                                             .format(serial, stop, step))
 
     def getACSLabel(self):
@@ -1209,7 +1115,7 @@ class AtomGroup(Atomic):
         """Set active coordinate set label."""
 
         if self._n_csets:
-            if isinstance(label, (str, NoneType)):
+            if label is None or isinstance(label, str):
                 self._cslabels[self._acsi] = label 
             else:
                 raise TypeError('label must be a string')
@@ -1225,7 +1131,8 @@ class AtomGroup(Atomic):
         
         if isinstance(labels, list):
             if len(labels) == self._n_csets:
-                if all(isinstance(lbl, (str, NoneType)) for lbl in labels):
+                if all((lbl is None or isinstance(lbl, str)) 
+                       for lbl in labels):
                     self._cslabels = list(labels)
                 else:
                     raise ValueError('all items of labels must be strings')
@@ -1364,3 +1271,88 @@ class AtomGroup(Atomic):
                 c += 1
         self._data['fragindex'] = fragindices
         self._fragments = fragments
+
+    
+for fname, field in ATOMIC_FIELDS.items():
+
+    meth = field.meth_pl
+    getMeth = 'get' + meth
+    setMeth = 'set' + meth
+    if field.call:
+        # Define public method for retrieving a copy of data array
+        if not field.private:
+            def getData(self, var=fname, call=field.call):
+                try:
+                    return self._data[var].copy()
+                except KeyError:
+                    [getattr(self, meth)() for meth in call]
+                    return self._data[var].copy()
+        # Define private method for retrieving actual data array
+        def _getData(self, var=fname, call=field.call):
+            try:
+                return self._data[var]
+            except KeyError:
+                [getattr(self, meth)() for meth in call]
+                return self._data[var].copy()
+    else:
+        if not field.private:
+            def getData(self, var=fname):
+                try:
+                    return self._data[var].copy()
+                except KeyError: 
+                    pass
+        def _getData(self, var=fname):
+            return self._data.get(var)
+
+    if not field.private:
+        getData = wrapGetMethod(getData)
+        getData.__name__ = getMeth
+        getData.__doc__ = field.getDocstr('get')
+        setattr(AtomGroup, getMeth, getData)
+    
+    _getData = wrapGetMethod(_getData)
+    _getData.__name__ = '_' + getMeth
+    _getData.__doc__ = field.getDocstr('_get')
+    setattr(AtomGroup, '_' + getMeth, _getData)
+    
+    if field.readonly or field.private:
+        continue
+    
+    # Define public method for setting values in data array
+    def setData(self, array, var=fname, dtype=field.dtype, 
+                ndim=field.ndim, none=field.none, flags=field.flags):
+        if array is None:
+            self._data.pop(var, None)
+        else:
+            if self._n_atoms == 0:
+                self._n_atoms = len(array)
+            elif len(array) != self._n_atoms:
+                raise ValueError('length of array must match number '
+                                   'of atoms')
+                
+            if isinstance(array, list):
+                array = np.array(array, dtype)
+            elif not isinstance(array, np.ndarray):
+                raise TypeError('array must be an ndarray or a list')
+            elif array.ndim != ndim:
+                    raise ValueError('array must be {0} '
+                                       'dimensional'.format(ndim))
+            elif array.dtype != dtype:
+                try:
+                    array = array.astype(dtype)
+                except ValueError:
+                    raise ValueError('array cannot be assigned type '
+                                     '{0}'.format(dtype))
+            self._data[var] = array
+            if none: self._none(none)
+            if flags and self._flags:
+                self._resetFlags(var)
+            
+    setData = wrapSetMethod(setData)
+    setData.__name__ = setMeth 
+    setData.__doc__ = field.getDocstr('set')
+    setattr(AtomGroup, setMeth, setData)
+
+del getData
+del _getData
+del setData
